package at.ac.tuwien.ims.acto_framework;

/**
 * TUIO <p>
 * 		data class for all data of a TUIO
 * 
 * @author Emanuel Vonach
 *
 */

import org.apache.commons.math3.complex.Quaternion;
import org.apache.commons.math3.geometry.euclidean.threed.Vector3D;

import android.util.Log;

//import android.util.Log;

public class TUIO {

	private final String TAG = this.getClass().getSimpleName();
	
	public byte id;					// the id of the TUIO the data is valid for
	public byte cosId;				// id of related coordinate system
	
	private TUIOPlayground playground;
	
	private ACTOData playPose;
	//public ACTOData worldPose;
	
	private Vector3D playTransVector;
	private Quaternion playRotQuaternion;
	
	private Vector3D destinationVector;
	private Quaternion destinationRotQuaternion;
	
//	public boolean online = false;
//	public boolean visible = false;
	public boolean isDriving = false;
	public boolean isOnRoute = false;

//	public float[] translation;		// = new float[3];
//	public float[] rotation;		// = new float[4];	(quaternion)
	
	public TUIO(byte id, byte cosId, TUIOPlayground playground) {
		this.id = id;
		this.cosId = cosId;
		this.playground = playground;
		
		playPose = new ACTOData(ACTOData.POSE, id, new float[] {0,0,0}, new float[] {0,0,0,0});
		//worldPose = new ACTOData(ACTOData.POSE, cosId, new float[3], new float[4]);
		
		playTransVector = new Vector3D((double)playPose.translation[0],(double)playPose.translation[1],(double)playPose.translation[2]);
		playRotQuaternion = new Quaternion((double)playPose.rotation[3],(double)playPose.rotation[0],(double)playPose.rotation[1],(double)playPose.rotation[2]);
	}
	
	@Override
    public boolean equals(Object compareObj)
    {
		if (this == compareObj) // Are they exactly the same instance?
	           return true;
	 
		if (compareObj == null) // Is the object being compared null?
		    return false;
	 
		if (!(compareObj instanceof TUIO)) // Is the object being compared also a Person?
		    return false;
	 
		TUIO compareTuio = (TUIO)compareObj; // Convert the object to a TUIO
	 
		return (this.id == compareTuio.id || this.cosId == compareTuio.cosId); // Are they equal?
    }
 
    @Override
    public int hashCode()
    {
		int primeNumber = 31;
		return primeNumber + this.id + this.cosId;
    }



	public void updateByWorldPose(ACTOData worldPose) {
		
//		Log.d(TAG, "TT (TU_WP): " + worldPose.translation[0]);	
		
		float [] tmpTrans = worldPose.translation;
		tmpTrans[0] = ((tmpTrans[0] + (playground.trackingRes[0]/2))/playground.trackingRes[0])*playground.resolution[0];
		tmpTrans[1] = ((tmpTrans[1] + (playground.trackingRes[1]/2))/playground.trackingRes[1])*playground.resolution[1];
		
		tmpTrans[2] = tmpTrans[2]-(float)playground.calibVector.getZ();
		
		playPose.translation = tmpTrans;//Arrays.copyOf(currTUIO.worldPose.translation, 3);		
		
//		playPose.translation = worldPose.translation;//Arrays.copyOf(currTUIO.worldPose.translation, 3);
		playPose.rotation =  worldPose.rotation;//Arrays.copyOf(currTUIO.worldPose.rotation, 3);
		
		playTransVector = new Vector3D((double)playPose.translation[0],(double)playPose.translation[1],(double)playPose.translation[2]);
		playRotQuaternion = new Quaternion((double)playPose.rotation[3],(double)playPose.rotation[0],(double)playPose.rotation[1],(double)playPose.rotation[2]);
		
//		Log.d(TAG, "TT (TU_PL): " + playPose.translation[0]);		
	}
	
	public ACTOData getPlayPose() {
		return playPose;
	}
	
	public Vector3D getTranslationVector() {
		return playTransVector;
	}
	
	public Quaternion getRotationQuaternion() {
		return playRotQuaternion;
	}
	
//	public ACTOData getDestinationPose() {
//		return playPose;
//	}
	public void driveTo(ACTOData destPose) {
		destinationVector = new Vector3D((double)destPose.translation[0],(double)destPose.translation[1],(double)destPose.translation[2]);
		destinationRotQuaternion = new Quaternion((double)destPose.rotation[3],(double)destPose.rotation[0],(double)destPose.rotation[1],(double)destPose.rotation[2]);
		isDriving = true;
		isOnRoute = false;
	}
	
	public Vector3D getDestinationVector() {
		return destinationVector;
	}
	
	public Quaternion getDestinationQuaternion() {
		return destinationRotQuaternion;
	}
}
