package at.ac.tuwien.ims.acto_framework;

//import java.io.IOException;
import java.util.ArrayList;
import java.util.List;

import android.app.Notification;
import android.app.NotificationManager;
import android.app.PendingIntent;
import android.app.Service;
//import android.content.ComponentName;
import android.content.Context;
import android.content.Intent;
import android.graphics.Point;
//import android.content.ServiceConnection;
import android.hardware.usb.UsbManager;
import android.net.wifi.WifiConfiguration;
import android.net.wifi.WifiManager;
import android.os.Bundle;
import android.os.Handler;
//import android.os.HandlerThread;
import android.os.IBinder;
//import android.os.Looper;
import android.os.Message;
import android.os.Messenger;
import android.os.RemoteException;
//import android.support.v4.app.NotificationCompat;
import android.text.format.Formatter;
import android.util.Log;
import android.view.Display;
//import android.widget.Toast;
import android.view.WindowManager;

/**
 * TUIOServer:
 * 		core service of ACTOframework
 * 
 * @author Emanuel Vonach
 */
public class TUIOServer extends Service {
	
	private final String TAG = this.getClass().getSimpleName();
	
	// For showing and hiding our notification.
    private NotificationManager mNM;
    private static int NOTIFY_ID = R.string.notification_srv_label;
	
	// Keeps track of all current registered clients.
    ArrayList<Messenger> mClients = new ArrayList<Messenger>(); 

    // The published target for clients to send messages to IncomingHandler.
    final Messenger mIncomingMessenger = new Messenger(new IncomingHandler());
    
	//define service behavior:
//	static final int mStartMode = START_STICKY; // only if also started, available: START_NOT_STICKY, START_STICKY, START_REDELIVER_INTENT
//	static final boolean mAllowRebind = false;	// do we need to rebind?
	
    
    // global instances helper classes
    private TUIOPlayground playground;
    //boolean play_mBound = false;

    private TUIOCommHub commHub;
    //boolean comm_mBound = false;
    
    //keep track of settings
	int usbConnected = ACTOConst.STATE_CLOSED;
	int appBroadcasting = ACTOConst.STATE_CLOSED;
	int wifiConnected = ACTOConst.STATE_CLOSED;
	int wifiBroadcasting = ACTOConst.STATE_CLOSED;
	int calibValid = ACTOConst.STATE_CLOSED;
    
	// const values for asking from outside
	static final int USB_CONNECTED = 0;
	static final int APP_BROADCASTING = 1;
	static final int WIFI_CONNECTED = 2;
	static final int WIFI_BROADCASTING = 3;
	static final int CALIB_VALID = 4;
	



    

    
    
 
	//
	// LIFECYCLE BEGIN
	/////////////////////
	
	@Override
	  public void onCreate() {
		//The system calls this method when the service is first created (before it calls either onStartCommand() or onBind()).
		//If the service is already running, this method is not called.
				
        Log.d(TAG, "onCreate");
        
        //TODO: look into super call and saved instance
        //super.onCreate(savedInstanceState);

        // start status notification
        showNotification();

        // TODO: probably also start, if getting killed after longer run
//        startForeground(NOTIFY_ID,notification);
        
        //TODO: probably dim screen, too bright now
        
        //initialize TUIO handling + CosID zuordnung von xml
        
        //prepare communication hub
        commHub = new TUIOCommHub(this);
        
        //prepare TUIO handling, coordinate conversion, etc
        playground = new TUIOPlayground(this);
        setDefPlaygroundSize();
        
        //check ob USB, frage nach bzw. starte serial connection
        
        //check ob WLAN gewnscht, Broadcast?
        
        //starte Kalibrierung
        
        //Main loop?
	}
	
//    @Override
//    public int onStartCommand(Intent intent, int flags, int startId) {
//    	//Usually, a started service performs a single operation and does not return a result to the caller.
//    	//When the operation is done, the service should stop itself.
//        Log.d(TAG, "onStartCommand, start id: " + startId + "+ intent: " + intent);
//
//        return mStartMode;
//    }
    
    @Override
	public IBinder onBind(Intent intent) {
    	// offers a client-server interface that allows components to bind with the service
    	// only called when first Components binds!
    	Log.d(TAG, "onBind, intent: " + intent);

        return mIncomingMessenger.getBinder();
	}
	
//    @Override
//    public boolean onUnbind(Intent intent) {
//        // All clients have unbound with unbindService()
//        return mAllowRebind;
//    }
//    
//    @Override
//    public void onRebind(Intent intent) {
//        // A client is binding to the service with bindService(),
//        // after onUnbind() has already been called
//    }
   
    @Override
    public void onDestroy() {
    	//Your service should implement this to clean up any resources such as threads, registered listeners, receivers, etc.
        
    	stopPlayground();
    	stopCommHub();
    	// Cancel the persistent notification.
        mNM.cancel(NOTIFY_ID);

    	Log.d(TAG, "service destroyed"); 
    }
    
	//
	// LIFECYCLE END
	/////////////////////

    // Show a notification while this service is running.	//TODO: How to handle PendingIntent from other App?
    private void showNotification() {
        mNM = (NotificationManager)getSystemService(Context.NOTIFICATION_SERVICE);

        //TODO: set Notification & Intent globally, so we can change it according to state
        // Set the icon, scrolling text and timestamp
        Notification notification = new Notification(R.drawable.ic_notification, getText(R.string.notification_srv_running), System.currentTimeMillis());

        // The PendingIntent to launch our activity if the user selects this notification
        PendingIntent contentIntent = PendingIntent.getActivity(this, 0, new Intent(this, ACTOGUI.class), 0);

        // Set the info for the views that show in the notification panel.
        notification.setLatestEventInfo(this, getText(R.string.notification_srv_label), getText(R.string.notification_srv_running), contentIntent);

        // Send the notification.
        // We use a string id because it is a unique number.  We use it later to cancel.
        mNM.notify(NOTIFY_ID, notification);
    }
    
    private void setDefPlaygroundSize() {
	    WindowManager wm = (WindowManager) this.getApplicationContext().getSystemService(Context.WINDOW_SERVICE);
	    Display display = wm.getDefaultDisplay();
	    Point size = new Point();
	    display.getSize(size);
//	    int width = display.getWidth();  // deprecated
//	    int height = display.getHeight();  // deprecated
	    //TODO: ACHTUNG X & Y VERTAUSCHT!!
	    playground.setSize(new int[] {size.y, size.x, 200});
    }
    
    //get local IP address (has to be accessed over service)
    protected String getLocalIpAddress()
 	{
 	    try 
 	    {
			WifiManager wim = (WifiManager)getSystemService(WIFI_SERVICE);
			//List<WifiConfiguration> l =  wim.getConfiguredNetworks(); 
			//WifiConfiguration wc = l.get(0);
			return Formatter.formatIpAddress(wim.getConnectionInfo().getIpAddress());
 	    	  
// 	        for (Enumeration<NetworkInterface> en = NetworkInterface.getNetworkInterfaces(); en.hasMoreElements();)
// 	        {
// 	            NetworkInterface intf = en.nextElement();
// 	            for (Enumeration<InetAddress> enumIpAddr = intf.getInetAddresses(); enumIpAddr.hasMoreElements();)
// 	            {
// 	                InetAddress inetAddress = enumIpAddr.nextElement();
// 	                if (!inetAddress.isLoopbackAddress()) 
// 	                {
// 	                    return inetAddress.getHostAddress().toString();
// 	                }
// 	            }
// 	        }
 	    } 
 	    catch (Exception ex)
 	    {
 	        Log.e(TAG, ex.toString());
 	    }
 	    return "";
 	}

    //
    //	State management
    ///////////////////////////////
 	public int getState(int constValue) {
		switch(constValue) {
			case USB_CONNECTED:
				return usbConnected;
			case APP_BROADCASTING:
				return appBroadcasting;
			case WIFI_CONNECTED:
				return wifiConnected;
			case WIFI_BROADCASTING:
				return wifiBroadcasting;
			case CALIB_VALID:
				return calibValid;
			default:
				Log.d(TAG, "UNKNOWN SETTING");
				return ACTOConst.STATE_UNDEFINED;
		}
 	}
 	
 	protected void setUsbState(int state) {
 		
 		//if STATE_UNDEFINED = toggle
 		if (state == ACTOConst.STATE_UNDEFINED) {
 			// if usb not established, try to establish
 			if (usbConnected == ACTOConst.STATE_CLOSED) state = ACTOConst.STATE_PENDING;
 			else state = ACTOConst.STATE_CLOSED;	//if usb pending or established, shutdown properly
 		}
 		
		if (state == ACTOConst.STATE_CLOSED) {
    		//if USB pending or established, shutdown properly
    		commHub.stopUsb();
    	} else {
    		// PENDING, CONNECTED
    		if (usbConnected == ACTOConst.STATE_CLOSED) {
    			
    			UsbManager mUsbManager = (UsbManager) getSystemService(Context.USB_SERVICE);
    			
    			commHub.startUsb(mUsbManager);
    		}
    		else {
    			//TODO: usb already connected, what is best action??
    		}
    	}
 	}
 	
 	protected void setAppBrcState(int state) {
 		
 		//if STATE_UNDEFINED = toggle
 		if (state == ACTOConst.STATE_UNDEFINED) {
 			// if Broadcast not established, try to establish
 			if (appBroadcasting == ACTOConst.STATE_CLOSED) state = ACTOConst.STATE_PENDING;
 			else state = ACTOConst.STATE_CLOSED;	//if Broadcast pending or established, stop it
 		}
 		
		if (state == ACTOConst.STATE_CLOSED) {
    		//if Broadcast pending or established, stop it
			appBroadcasting = ACTOConst.STATE_CLOSED;
    	} else {
    		// PENDING, CONNECTED
    		appBroadcasting = ACTOConst.STATE_CONNECTED;	     	
    	}
		
		sendToSuperActivity(ACTOConst.SVC_SET_SETTING, APP_BROADCASTING, appBroadcasting);
		//commHub.wifiSendState(APP_BROADCASTING, appBroadcasting);
 	}
		    	
 	protected void setWifiState(int statePort) {
 		//TODO: abfrage ob HW-Wifi berhaupt an

 		//if STATE_UNDEFINED = toggle
 		if (statePort == ACTOConst.STATE_UNDEFINED) {
 			if (wifiConnected == ACTOConst.STATE_CLOSED) statePort = ACTOConst.STATE_PENDING;
 			else statePort = ACTOConst.STATE_CLOSED;	//if wifi pending or established, shutdown properly
 		}

 		if (statePort == ACTOConst.STATE_CLOSED) {
    		commHub.stopWifi();
    	} else {
    		//PENDING, CONNECTED, port
    		if (wifiConnected == ACTOConst.STATE_CLOSED) {
	 	 		// if >5 it is most likely a port
	 			if (statePort >= 5)	commHub.startWifi(statePort);
	 			else commHub.startWifi(ACTOConst.STATE_UNDEFINED);
    		}
    		else {
    			//wifi is at least pending, but listening at wrong port
    			if ((statePort >= 5) && (commHub.getPort() != statePort)) {
    				//re-establish wifi with new port
    				commHub.startWifi(statePort);
    			}
    		}
    		
 			//tell the activity
        	Bundle wifiInfo = new Bundle();
        	wifiInfo.putString("IP", getLocalIpAddress());
        	wifiInfo.putInt("port", commHub.getPort());
        	sendToSuperActivity(ACTOConst.SVC_INFO_WIFI, wifiInfo);
    	}
 	}

 	protected void setWifiBrcState(int state) {
 		
 		//if STATE_UNDEFINED = toggle
 		if (state == ACTOConst.STATE_UNDEFINED) {
 			// if Broadcast not established, try to establish
 			if (wifiBroadcasting == ACTOConst.STATE_CLOSED) state = ACTOConst.STATE_PENDING;
 			else state = ACTOConst.STATE_CLOSED;	//if Broadcast pending or established, stop it
 		}
 		
		if (state == ACTOConst.STATE_CLOSED) {
    		//if Broadcast pending or established, stop it
			wifiBroadcasting = ACTOConst.STATE_CLOSED;
    	} else {
    		// PENDING, CONNECTED
	     	if (wifiConnected == ACTOConst.STATE_CONNECTED) wifiBroadcasting = ACTOConst.STATE_CONNECTED;
	     	else wifiBroadcasting = ACTOConst.STATE_PENDING;
    	}
		
		sendToSuperActivity(ACTOConst.SVC_SET_SETTING, WIFI_BROADCASTING, wifiBroadcasting);
		commHub.wifiSendState(WIFI_BROADCASTING, wifiBroadcasting);
 	}

 	protected void setCalibState(int state) {

 		//if STATE_UNDEFINED = toggle
 		if (state == ACTOConst.STATE_UNDEFINED) {
 			// if no calibration valid yet, do one
 			if (calibValid == ACTOConst.STATE_CLOSED) state = ACTOConst.STATE_PENDING;
 			else state = ACTOConst.STATE_CLOSED;	//if calibration already pending or established, reset it
 		}
 		
 		playground.resetCalibration();
 		
		if (state != ACTOConst.STATE_CLOSED) {
			// PENDING, CONNECTED, establish calibration
			playground.startCalibration();
    	}
		//else calibration is already reset anyway 
 	}

    //
    //	State management END
    ///////////////////////////////
 	
 	
    //
    //	Tracking data management 
    ///////////////////////////////
 	
 	//called by playground to broadcast pose update
 	public void broadcastTuioPose(ACTOData pose) {
     	if (wifiBroadcasting == ACTOConst.STATE_CONNECTED) {
     		commHub.wifiSendTuioPose(pose);
     	}
     	if (appBroadcasting == ACTOConst.STATE_CONNECTED) {
     		sendToSuperActivity(ACTOConst.SVC_SET_POSE, ACTOData.ADataToBundle(pose));
     	}
	}
 	
 	//called by playground to send pose update
 	//TODO: bad hack, implement properly after async reconstruction of all calls 
 	public void badTuioPoseHack(ACTOData pose) {
 		commHub.wifiSendTuioPose(pose);
 		sendToSuperActivity(ACTOConst.SVC_SET_POSE, ACTOData.ADataToBundle(pose));
	}
 	
 	
//	protected ACTOData getTuioState(byte id) {
//		return null;
//		//inExtMessage, T, R (return AD)
//	}
		
	protected boolean getAllPoseUpdate() {
		return playground.getAllPoseUpdate();
	}
 	
	protected ACTOData getTuioPose(byte id) {
		return playground.getTuioPose(id);
	}
	protected boolean getTuioExt(byte usbMsg, ACTOData ext) {
		return commHub.usbSendExt(usbMsg, ext);
		//can only return ACTOData directly, if data used from TUIO in Database! faster? reliable?
	}
	protected ACTOData getTuioTrans(byte id) {
		return playground.getTuioTrans(id);
 	}
 	
	protected ACTOData getTuioRot(byte id) {
		return playground.getTuioRot(id);
 	}
 	
	protected int[] getPlaygroundSize() {	//TODO: as ACTOData?
		return playground.getSize();
	}
	
//	protected void setTuioState(ACTOData state) {
//		//inExtMessage, T, R
//	}
	
	protected void setTuioPose(ACTOData pose) {
		playground.setTuioPose(pose);
	}
	
	//TODO:same as getTuioExt!!
	protected boolean setTuioExt(byte usbMsg, ACTOData ext) {
		return commHub.usbSendExt(usbMsg, ext);
	}
	
	protected void setTuioTrans(ACTOData trans) {
		playground.setTuioTrans(trans);
 	}
 	
	protected void setTuioRot(ACTOData rot) {
		playground.setTuioRot(rot);
 	}

	protected void setPlaygroundSize(int[] res) {
		playground.setSize(res);
	}

 	//TODO: FOR TESTING
	protected void testSth(int one, int two) {
		playground.testSth();
	}
 	
 	
//	protected void moveListener() {
//		//ID, T, R
//	}
//	protected void extStateListener() {
//		// ID, EXTSTATE
//	}
 	
    //
    //	Tracking data management END
    ///////////////////////////////
 	
 	
 	
 	//
 	// Feedback from CommHub
 	///////////////////////
 	protected void onWifiStateChange(int state) {

    	wifiConnected = state;
    	//tell Activity new state
     	sendToSuperActivity(ACTOConst.SVC_SET_SETTING, WIFI_CONNECTED, wifiConnected);
     	commHub.wifiSendState(WIFI_CONNECTED, wifiConnected);
     	
     	//if wifi active, also activate broadcast button
     	if (wifiConnected == ACTOConst.STATE_CONNECTED) {
     		if (wifiBroadcasting == ACTOConst.STATE_PENDING) {
     			wifiBroadcasting = ACTOConst.STATE_CONNECTED;
     			sendToSuperActivity(ACTOConst.SVC_SET_SETTING, WIFI_BROADCASTING, wifiBroadcasting);
     			commHub.wifiSendState(WIFI_BROADCASTING, wifiBroadcasting);
     		}
     	}
     	else {
     		if (wifiBroadcasting == ACTOConst.STATE_CONNECTED) {
     			wifiBroadcasting = ACTOConst.STATE_PENDING;
     			sendToSuperActivity(ACTOConst.SVC_SET_SETTING, WIFI_BROADCASTING, wifiBroadcasting);
     			commHub.wifiSendState(WIFI_BROADCASTING, wifiBroadcasting);
     		}
     	}
    }
    
 	protected void onUsbStateChange(int state) {
 		//TODO: also send confirmation to wifi, if active
    	usbConnected = state;
     	sendToSuperActivity(ACTOConst.SVC_SET_SETTING, USB_CONNECTED, usbConnected);
     	commHub.wifiSendState(USB_CONNECTED, usbConnected);
    }
    
 	protected void onUsbMsg(byte usbMsg, ACTOData ext) {
 		//answer from usb, send to all interested parties, Playground, APP, WIFI
 		//playground.??
     	sendToSuperActivity(ACTOConst.SVC_SET_EXT, usbMsg, ACTOData.ADataToBundle(ext));
     	commHub.wifiSendTuioExt(usbMsg, ext);
    }
 	
 	//
 	// Feedback from CommHub END
 	///////////////////////
    
 	protected void onCalibStateChange(int state) {
    	calibValid = state;
     	sendToSuperActivity(ACTOConst.SVC_SET_SETTING, CALIB_VALID, calibValid);
     	commHub.wifiSendState(CALIB_VALID, calibValid);
    }

    
    // shutdown commHub properly
    private void stopCommHub() {

    	try {commHub.close();}
    	catch (Exception ex) {}
    	commHub = null;
    }
    
    // shutdown playground properly
    private void stopPlayground() {

    	try {playground.close();}
    	catch (Exception ex) {}
    	playground = null;
    }
    

    
    

    
    
  
    //
	// SERVICE COMMUNICATION (INCOMING)
	////////////////  

     // Handler of INCOMING messages from clients.
     class IncomingHandler extends Handler {
     	
         @Override
         public void handleMessage(Message incomingMsg) {
         	// INCOMING messages are handled here!
         	switch (incomingMsg.what) {
         		case ACTOConst.SVC_REG_CLIENT:
         			mClients.add(incomingMsg.replyTo);
         			//TODO: add to Log
         			Log.d(TAG, "REGISTERING!!!");
         			
         			sendToSuperActivity(ACTOConst.SVC_INIT_DONE, 0, 0);
         			
         			//set correct states in new client
         	    	sendToSuperActivity(ACTOConst.SVC_SET_SETTING, USB_CONNECTED, usbConnected);
         	    	sendToSuperActivity(ACTOConst.SVC_SET_SETTING, APP_BROADCASTING, appBroadcasting);
         	    	sendToSuperActivity(ACTOConst.SVC_SET_SETTING, WIFI_CONNECTED, wifiConnected);
         	    	sendToSuperActivity(ACTOConst.SVC_SET_SETTING, WIFI_BROADCASTING, wifiBroadcasting);
         	    	sendToSuperActivity(ACTOConst.SVC_SET_SETTING, CALIB_VALID, calibValid);
         	    	
         			break;
         			
         		case ACTOConst.SVC_UNREG_CLIENT:
         			mClients.remove(incomingMsg.replyTo);
         			//TODO: add to Log
                     Log.d(TAG, "UNREGISTERING!!!");
         			break;
         		
     			//TODO: evt. if started, e.g. display off
//             	case SVC_KILL_SERVICE:
//             		mClients.remove(msg.replyTo);
//             		stopSelf();
//             		break;
         			
     			// Activity asks for sync of setting
         		case ACTOConst.SVC_GET_SETTING:
         			switch(incomingMsg.arg1) {
         				case USB_CONNECTED:
         					sendToSuperActivity(ACTOConst.SVC_SET_SETTING, USB_CONNECTED, usbConnected);
         					break;
         				case APP_BROADCASTING:
         					sendToSuperActivity(ACTOConst.SVC_SET_SETTING, APP_BROADCASTING, appBroadcasting);
         					break;
         				case WIFI_CONNECTED:
         					sendToSuperActivity(ACTOConst.SVC_SET_SETTING, WIFI_CONNECTED, wifiConnected);
         					break;
         				case WIFI_BROADCASTING:
         					sendToSuperActivity(ACTOConst.SVC_SET_SETTING, WIFI_BROADCASTING, wifiBroadcasting);
         					break;
         				case CALIB_VALID:
         					sendToSuperActivity(ACTOConst.SVC_SET_SETTING, CALIB_VALID, calibValid);
         					break;
         				default:
         					Log.d(TAG, "UNKNOWN SETTING");
							sendToSuperActivity(ACTOConst.SVC_SET_SETTING, incomingMsg.arg1, ACTOConst.STATE_UNDEFINED);
         			}
         			break;
         			
         		// Activity wants settings changed (e.g. on button click)
         		case ACTOConst.SVC_SET_SETTING:
					switch(incomingMsg.arg1) {
						case USB_CONNECTED:
							setUsbState(incomingMsg.arg2);
							break;
						case APP_BROADCASTING:
							setAppBrcState(incomingMsg.arg2);
							break;
						case WIFI_CONNECTED:
							setWifiState(incomingMsg.arg2);
							break;
						case WIFI_BROADCASTING:
							setWifiBrcState(incomingMsg.arg2);
							break;
						case CALIB_VALID:
							setCalibState(incomingMsg.arg2);
							break;
						default:
							Log.d(TAG, "UNKNOWN SETTING");
							sendToSuperActivity(ACTOConst.SVC_SET_SETTING, incomingMsg.arg1, ACTOConst.STATE_UNDEFINED);			
					}
	     			break;
         			
	     			
	     			
//     			//TODO: implement when data structure ready
// 				// simple CMD messages
// 				case (ACTOConst.SVC_FULL_STATE): case (ACTOConst.SVC_FULL_EXT):
// 				case (ACTOConst.SVC_FULL_TRANS): case (ACTOConst.SVC_FULL_ROT): 
         		
         		case (ACTOConst.SVC_ALL_POSE):
         			getAllPoseUpdate();
 					break;
	     				
 				
 				
	     		// read CMD + ID messages
//				case (ACTOConst.SVC_GET_STATE):
//					//TODO: check if cast to byte is ok & understood!
//					sendToSuperActivity(ACTOConst.SVC_SET_STATE, ACTOData.ADataToBundle(getTuioState((byte)incomingMsg.arg1)));
//					break;

				case (ACTOConst.SVC_GET_POSE):
					sendToSuperActivity(ACTOConst.SVC_SET_POSE, ACTOData.ADataToBundle(getTuioPose((byte)incomingMsg.arg1)));
					break;

				case (ACTOConst.SVC_GET_EXT):
					getTuioExt((byte)incomingMsg.arg1, ACTOData.BundletoAData(incomingMsg.getData()));
					// TODO: HAS TO BE ASYNC? OR FROM DATABASE, see getTuioExt
					break;

				case (ACTOConst.SVC_GET_TRANS):
					sendToSuperActivity(ACTOConst.SVC_SET_TRANS, ACTOData.ADataToBundle(getTuioTrans((byte)incomingMsg.arg1)));
					break;

				case (ACTOConst.SVC_GET_ROT):
					sendToSuperActivity(ACTOConst.SVC_SET_ROT, ACTOData.ADataToBundle(getTuioRot((byte)incomingMsg.arg1)));
					break;
	     					
				case (ACTOConst.SVC_GET_PLSIZE):
					Bundle plSize = new Bundle();
					plSize.putByte(ACTOData.B_TYP, ACTOData.RES);
					plSize.putIntArray(ACTOData.B_RES, getPlaygroundSize());

					sendToSuperActivity(ACTOConst.SVC_SET_PLSIZE, plSize);
					break;
					
//				case (ACTOConst.SVC_SET_STATE):
// 					setTuioState(ACTOData.BundletoAData(incomingMsg.getData()));
//	     			break;

				case (ACTOConst.SVC_SET_POSE):
					setTuioPose(ACTOData.BundletoAData(incomingMsg.getData()));
     				break;
	     				
				case (ACTOConst.SVC_SET_EXT):
					setTuioExt((byte)incomingMsg.arg1, ACTOData.BundletoAData(incomingMsg.getData()));
	     			break;

 				case (ACTOConst.SVC_SET_TRANS):
					setTuioTrans(ACTOData.BundletoAData(incomingMsg.getData()));
     				break;

 				case (ACTOConst.SVC_SET_ROT):
					setTuioRot(ACTOData.BundletoAData(incomingMsg.getData()));
     				break;
 				
 				// CMD + int[] messages
 				case (ACTOConst.SVC_SET_PLSIZE): 					
 					setPlaygroundSize(incomingMsg.getData().getIntArray(ACTOData.B_RES));
 					break;

         			//incoming tracking update
         		case ACTOConst.SVC_TRK_UPDATE:

                 	ACTOData update = ACTOData.BundletoAData(incomingMsg.getData());
                 	
//                 	Log.d(TAG, "TT (TS): " + update.translation[0]);		
                 	
                 	playground.onNewTrackingData(update);
                 	break;
         			
         		case ACTOConst.SVC_TEST_STH:
         			testSth(incomingMsg.arg1, incomingMsg.arg2);
         			break;
         			
                 default:
                	 Log.d(TAG, "MESSAGE UNKNOWN: " + incomingMsg.what);
                     super.handleMessage(incomingMsg);
             }
         }
     }
         
     
     //
 	// SERVICE COMMUNICATION END
 	////////////////   
 



     //
     // SERVICE COMMUNICATION (OUTGOING) to ACTOActivity
     ///////////////////////////

     // simple Msg and/or int values
     private void sendToSuperActivity(int outgoingMsgCode, int outgoingValue1, int outgoingValue2) {
     	
     	Message ougoingMsg = Message.obtain(null, outgoingMsgCode, outgoingValue1, outgoingValue2);
     	
     	// cycle through all registered clients
 		for (int i=mClients.size()-1; i>=0; i--) {
 		    try {
 		        mClients.get(i).send(ougoingMsg);
 		    } catch (RemoteException e) {
 		        // The client is dead.  Remove it from the list;
 		        // we are going through the list from back to front
 		        // so this is safe to do inside the loop.
 		        mClients.remove(i);
 		    }
 		}
     }
     
     // more complex data
     private void sendToSuperActivity(int outgoingMsgCode, Bundle outgoingData) {
     
     	Message ougoingMsg = Message.obtain(null, outgoingMsgCode, 0, 0);
     	ougoingMsg.setData(outgoingData);
     	
     	// cycle through all registered clients
 		for (int i=mClients.size()-1; i>=0; i--) {
 		    try {
 		        mClients.get(i).send(ougoingMsg);
 		    } catch (RemoteException e) {
 		        // The client is dead.  Remove it from the list;
 		        // we are going through the list from back to front
 		        // so this is safe to do inside the loop.
 		        mClients.remove(i);
 		    }
 		}
     }
     
     // usb data
     private void sendToSuperActivity(int outgoingMsgCode, int usbMsgCode, Bundle outgoingData) {
     
     	Message ougoingMsg = Message.obtain(null, outgoingMsgCode, usbMsgCode, 0);
     	ougoingMsg.setData(outgoingData);
     	
     	// cycle through all registered clients
 		for (int i=mClients.size()-1; i>=0; i--) {
 		    try {
 		        mClients.get(i).send(ougoingMsg);
 		    } catch (RemoteException e) {
 		        // The client is dead.  Remove it from the list;
 		        // we are going through the list from back to front
 		        // so this is safe to do inside the loop.
 		        mClients.remove(i);
 		    }
 		}
     }
     

     
}
