package at.ac.tuwien.ims.acto_framework;

/**
 * UsbSerialComm:
 * 		handles communication between Android and Arduino
 * 
 * @author Emanuel Vonach
 * 
 * based on code by Thomas Mayr, Mark Ormos, Mike Wakerly (opensource@hoho.com)
 */

import java.io.IOException;
import java.nio.ByteBuffer;
import java.nio.ByteOrder;
import java.util.concurrent.BlockingQueue;
import java.util.concurrent.LinkedBlockingQueue;

//import android.hardware.usb.UsbDevice;
import android.hardware.usb.UsbManager;
import android.os.SystemClock;
import android.util.Log;

import com.hoho.android.usbserial.driver.UsbSerialDriver;
import com.hoho.android.usbserial.driver.UsbSerialProber;
//import com.hoho.android.usbserial.util.HexDump;
//import com.hoho.android.usbserial.util.SerialInputOutputManager;
//import com.hoho.android.usbserial.util.SerialInputOutputManager.Listener;


//TODO: proper synchronization throughout the class!!
public class UsbSerialComm implements Runnable 
{
	private final String TAG = this.getClass().getSimpleName();
	
	 //calling commHub
    private TUIOCommHub commHub;
	
    private boolean reconnect = true;
    
    
    private boolean isListening = false;
    private boolean usbAccessGranted = false;  

	/**manager for the USB*/
	private UsbManager mUsbManager;
	private UsbSerialDriver driver = null;
	
	
    private static final int READ_WAIT_MILLIS = 100;
    private static final int BUFSIZ = 1024;

    private final ByteBuffer readBuffer = ByteBuffer.allocate(BUFSIZ);

    private final ByteBuffer writeBuffer = ByteBuffer.allocate(BUFSIZ);
    
    private final ByteBuffer interpreter = ByteBuffer.allocate(4);	//Scanner might also work
    

    
    public UsbSerialComm(UsbManager usbManager, TUIOCommHub commHub) throws IOException
	{
		Log.d(TAG,"starting UsbSerialComm");
		
		mUsbManager = usbManager;
		this.commHub = commHub;
		
		interpreter.order(ByteOrder.LITTLE_ENDIAN);
	}
    
    
	//function for thread access
	private void usbStateChange(int state) {
		commHub.onUsbStateChange(state);
	}
    
    
  //listen for clients and on input
  	public void run() 
  	{
  		
  		//if connection gets lost and reconnect is set true, retry to accept new clients
  		while(reconnect)
  		{
  			//wait for client
  			Log.d(TAG, "waiting for clients...");
  			//tell calling class that we are ready to accept clients
			usbStateChange(ACTOConst.STATE_PENDING);
  			
			//locks until driver is found or thread aborted
			while (driver == null && reconnect == true) {
				// Find the first available driver.
  				//driver = UsbSerialProber.acquire(mUsbManager);
  				driver = UsbSerialProber.findFirstDevice(mUsbManager);
  				SystemClock.sleep(1000);	//search once a second
			}
			//TODO: can also reach this, if reconnect changed to false, check that and below
			//TODO: integrate some permission mechanism
//			mUsbManager.requestPermission(driver, mPermissionIntent);
			// ...the call to requestPermission() is asynchronous. ... it shows the user a dialog and asks the user if he will grant permission to your application.
			// the PendingIntent that you passed to requestPermission() is used to broadcast an Intent indicating the permission was granted (or not).
			// You need to listen for this in a registered BroadcastReceiver and when the onReceive() method is called 
			// you can then examine the extras in the received Intent and decide how to proceed.
			// http://stackoverflow.com/questions/10994848/android-usb-enumeration
			
			Log.d(TAG, "waiting for user acknowledgement");
			while (usbAccessGranted == false && reconnect == true) {
				SystemClock.sleep(2000); //TODO only hack, use proper acceptance mechanism
				usbAccessGranted = true;
			}
			
  			try{
  				driver.open();
  				driver.setParameters(115200, UsbSerialDriver.DATABITS_8, UsbSerialDriver.STOPBITS_1, UsbSerialDriver.PARITY_NONE);
			}
  			catch (Exception e)
  			{
  				Log.e(TAG,"opening driver failed: " + e);
//  				e.printStackTrace();
  				//usbStateChange(ACTOConst.STATE_PENDING);
				//isListening = false;
  				//closeTransmitter();
  				continue; //found usb device caused a problem, skip the rest and try again
  			}
  			
  			//Connection is open now listen for input
  			Log.d(TAG,"Client connected");
			usbStateChange(ACTOConst.STATE_CONNECTED);
			
			//TODO: check thread security, synchronized??
  			isListening = true;

  			//listen on input
  			while (isListening) {

  				try
  				{
  					int numBytesRead = driver.read(readBuffer.array(), READ_WAIT_MILLIS);
  					
  					if (numBytesRead > 0) {
//  						Log.d(TAG, "num: " + numBytesRead);
  						
  					    final byte[] data = new byte[numBytesRead];
  					    readBuffer.get(data, 0, numBytesRead);
  					    
						for (int i = 0; i < numBytesRead; i++) {
							messageBuffer.add(data[i]);
						}
						checkForMessage();
  					    //updateReceivedData();
  					    //clear the buffer for next cycle
  					    readBuffer.clear();
  					}
  					
  					// Handle outgoing data.
  					int numBytesWrite = 0;
					byte[] outBuff = null;
					
					synchronized (writeBuffer) {
					    if (writeBuffer.position() > 0) {
					    	numBytesWrite = writeBuffer.position();
					        outBuff = new byte[numBytesWrite];
					        writeBuffer.rewind();
					        writeBuffer.get(outBuff, 0, numBytesWrite);
					        writeBuffer.clear();						//do something about ACK here!!
					       
//					        Log.d(TAG, "tell MOTOR (inBUF)");
					    }
					}
					if (outBuff != null) {
						driver.write(outBuff, READ_WAIT_MILLIS);
						
//				        Log.d(TAG, "TT US (after write)" + outBuff[2]);
				        
				        
						//TODO: only for debug, remove when not needed
//						for (int i = 0; i < numBytesWrite; i++) {  // i indexes each element successively.
//							Log.d(TAG, "WRITE USB: "+ i + ":" + outBuff[i]);
//						}
					}

  				}
  				catch(IOException e)
  				{
  					Log.e(TAG, "error in USB communication");
////  					//e.printStackTrace();
////  					isListening = false; //stop reading and try to reconnect
////  					cleanupStreams();
////  					break; 
//  					commHub.wifiSendByte(ACTOConst.WI_NAK);
  				}
  			}
  		}
  	}	
  		
  	
	//
	//	READING
	///////////////////////

    private BlockingQueue<Byte> messageBuffer = new LinkedBlockingQueue<Byte>();
    private boolean isCleaning = false;
    //TODO: include checksum in protocol if necessary
    //TODO: protokoll umsetzen
    //TODO: achtung spezial vorgehen wenn SEP als daten verwendet!! z.B. SEP+1 + SZ??
    
    
    private void checkForMessage() {

    	if (messageBuffer.isEmpty()) return;
    	else
    	{
        	if (isCleaning) {
        		doMessageBufferCleanup();
        		return;
        	}
	    	switch (messageBuffer.peek()) {
	    	
				case ACTOConst.USB_ACK:
					if(!readSimpleCmd()) return; //message is incomplete, wait for next cycle
					break;
	
				case ACTOConst.USB_PING: case ACTOConst.USB_SYNC:
					if (!readADId()) return; //message is not complete yet, wait for next cycle
					break;
					
					//makes no sense in that direction?
//				case ACTOConst.USB_GET_DI_UNIT: case ACTOConst.USB_GET_AN_UNIT: case ACTOConst.USB_GET_BYTE:
//				case ACTOConst.USB_GET_CHAR: case ACTOConst.USB_GET_INT: case ACTOConst.USB_GET_FLOAT:
//					if (!readADIdDid()) return; //message is not complete yet, wait for next cycle
//					break;
					
				//case ACTOConst.USB_MOTOR: case ACTOConst.USB_MOTOR_MS: //makes no sense in that direction?
					
				case ACTOConst.USB_SET_DI_UNIT: case ACTOConst.USB_SET_BYTE:
					if (!readADIdDidByte()) return; //message is not complete yet, wait for next cycle
					break;
					
				case ACTOConst.USB_SET_CHAR:
					if (!readADIdDidChar()) return; //message is not complete yet, wait for next cycle
					break;
					
				//case ACTOConst.USB_SET_PWM_UNIT: //makes no sense in that direction?
				case ACTOConst.USB_SET_AN_UNIT:	case ACTOConst.USB_SET_INT:
					if (!readADIdDidInt()) return; //message is not complete yet, wait for next cycle
					break;

				case ACTOConst.USB_SET_FLOAT:
					if (!readADIdDidFloat()) return; //message is not complete yet, wait for next cycle
					break;
				
				default:
					Log.e(TAG, "USB message corrupt, cleaning up buffer");
					sendByte(ACTOConst.USB_NAK);
					doMessageBufferCleanup();
	    	}
    	}
    	//if message was read successfully, there might be another message, so check again
    	checkForMessage();
    }
    
    
	/**
	 * Dump everything on USB until next end of message USB_SEP, to make sure the messageBuffer is clean.
	 * 	sends USB_NAK for dumped second message
	 * @param boolean isCleaning: is set accordingly
	 */
    //drop everything, until next end of message, to be sure our buffer is clean
    private void doMessageBufferCleanup() {
    	isCleaning = true;
    	interpreter.clear();
		try {
			while (!messageBuffer.remove().equals(ACTOConst.USB_SEP));
		}
		catch (Exception ex) {
			return;	//no end of message yet, continue in next cycle
		}
    	isCleaning = false;
    	sendByte(ACTOConst.USB_NAK);	//for the dropped next message
    	//worst case, by chance in msg USB_SEP occurs as data, then next part is corrupt and dropped as well 
    }

		
		//TODO: commHub could already be destroyed in the meantime (in reading function until handed over to commHub)
    
	/**
	 * Reads from USB messageBuffer: byte command,
	 * 	sends command to commHandler and USB_ACK
	 * @return true if successful, false if error occured
	 */
	private boolean readSimpleCmd() {							// format: CMD, SEP
		if (messageBuffer.size() >= 2) {
			final byte cmd;
			//TODO: do I want to know that in commHandler?
			cmd = messageBuffer.poll();
			
			if (messageBuffer.poll().equals(ACTOConst.USB_SEP)) {
				commHub.handleUsbMsg(cmd, null);
				sendByte(ACTOConst.USB_ACK);
				return true;
			}
			else {
				Log.e(TAG, "USB message corrupt, cleaning up buffer");
				sendByte(ACTOConst.USB_NAK);
				doMessageBufferCleanup();
			}
		}
		return false;
	}
		
		
	
	/**
	 * Reads from USB messageBuffer: byte command, byte id
	 * 	sends command & data to commHandler and USB_ACK/USB_NAK
	 * @return true if successful, false if error occured
	 */
	private boolean readADId() {								// format: CMD, ID, SEP
		
		if (messageBuffer.size() >= 3) {
			
			final byte cmd;
			final ACTOData dataPacket = new ACTOData(ACTOData.ID);

			cmd = messageBuffer.poll();
			dataPacket.id = messageBuffer.poll();

			if (messageBuffer.poll().equals(ACTOConst.USB_SEP)) {
				sendByte(ACTOConst.USB_ACK);
				commHub.handleUsbMsg(cmd, dataPacket);
				return true;
			}
			else {
				Log.e(TAG, "USB message corrupt, cleaning up buffer");
				sendByte(ACTOConst.USB_NAK);
				doMessageBufferCleanup();
			}
		}
		return false;
	}
	    
//	/**
//	 * Reads from USB messageBuffer: byte command, byte id
//	 * 	sends command & data to commHandler and USB_ACK/USB_NAK
//	 * @return true if successful, false if error occured
//	 */

	private boolean readADIdDid() {		//CMD, ID, dataID(byte), SEP
		if (messageBuffer.size() >= 4) {
			
			final byte cmd;
			final ACTOData dataPacket = new ACTOData(ACTOData.DATA_EMPTY);

			cmd = messageBuffer.poll();
			
			dataPacket.id = messageBuffer.poll();
			dataPacket.dataId = messageBuffer.poll();
			
			if (messageBuffer.poll().equals(ACTOConst.USB_SEP)) {
				sendByte(ACTOConst.USB_ACK);
				commHub.handleUsbMsg(cmd, dataPacket);
				return true;
			}
			else {
				Log.e(TAG, "USB message corrupt, cleaning up buffer");
				sendByte(ACTOConst.USB_NAK);
				doMessageBufferCleanup();
			}
		}
		return false;
	}
	
	private boolean readADIdDidByte() {		//CMD, ID, dataID(byte), data(byte), SEP	//for data & dig unit & motor
		if (messageBuffer.size() >= 5) {
			
			final byte cmd;
			final ACTOData dataPacket = new ACTOData(ACTOData.DATA_BYTE);

			cmd = messageBuffer.poll();
			
			dataPacket.id = messageBuffer.poll();
			dataPacket.dataId = messageBuffer.poll();
			dataPacket.bData = messageBuffer.poll();
			
			if (messageBuffer.poll().equals(ACTOConst.USB_SEP)) {
				sendByte(ACTOConst.USB_ACK);
				commHub.handleUsbMsg(cmd, dataPacket);
				return true;
			}
			else {
				Log.e(TAG, "USB message corrupt, cleaning up buffer");
				sendByte(ACTOConst.USB_NAK);
				doMessageBufferCleanup();
			}
		}
		return false;
	}
		
	private boolean readADIdDidChar() {		//CMD, ID, dataID(byte), data(char), SEP

		if (messageBuffer.size() >= 6) {
			
			final byte cmd;
			final ACTOData dataPacket = new ACTOData(ACTOData.DATA_CHAR);

			cmd = messageBuffer.poll();
			
			dataPacket.id = messageBuffer.poll();
			dataPacket.dataId = messageBuffer.poll();
			
			interpreter.clear();
			
			//next 2 bytes are char
			for (int i = 0; i < Character.SIZE/8; i++)
			interpreter.put(messageBuffer.poll());

//			interpreter.rewind();
//			Log.i(TAG, "oder: " + interpreter.get() + ", " + interpreter.get() + ", " + interpreter.get() + ", " + interpreter.get());

			interpreter.rewind();
			dataPacket.cData = interpreter.getChar();

			if (messageBuffer.poll().equals(ACTOConst.USB_SEP)) {
				sendByte(ACTOConst.USB_ACK);
				commHub.handleUsbMsg(cmd, dataPacket);
				return true;
			}
			else {
				Log.e(TAG, "USB message corrupt, cleaning up buffer");
				sendByte(ACTOConst.USB_NAK);
				doMessageBufferCleanup();
			}
		}
		return false;
	}
			
	private boolean readADIdDidInt() {		//CMD, ID, dataID(byte), data(int), SEP		//for data & ana unit
		if (messageBuffer.size() >= 8) {
			Log.i(TAG, "size: " + messageBuffer.size());
			final byte cmd;
			final ACTOData dataPacket = new ACTOData(ACTOData.DATA_INT);

			cmd = messageBuffer.poll();
			
			dataPacket.id = messageBuffer.poll();
			dataPacket.dataId = messageBuffer.poll();
			
			
			Log.i(TAG, "for id: " + dataPacket.id + " and dataid: " + dataPacket.dataId);
			
			interpreter.clear();
			
			//next 4 bytes are int
			for (int i = 0; i< Integer.SIZE/8; i++)
			interpreter.put(messageBuffer.poll());
			
			interpreter.rewind();
			dataPacket.iData = interpreter.getInt(); //reading in reverse (LITTLE_ENDIAN order)
			
			if (messageBuffer.poll().equals(ACTOConst.USB_SEP)) {
				sendByte(ACTOConst.USB_ACK);
				commHub.handleUsbMsg(cmd, dataPacket);
				return true;
			}
			else {
				Log.e(TAG, "USB message corrupt, cleaning up buffer");
				sendByte(ACTOConst.USB_NAK);
				doMessageBufferCleanup();
			}
		}
		return false;
	}
				
	private boolean readADIdDidFloat() {	//CMD, ID, dataID(byte), data(float), SEP
		if (messageBuffer.size() >= 8) {
			
			final byte cmd;
			final ACTOData dataPacket = new ACTOData(ACTOData.DATA_FLOAT);

			cmd = messageBuffer.poll();
			
			dataPacket.id = messageBuffer.poll();
			dataPacket.dataId = messageBuffer.poll();
			
			interpreter.clear();
			
			//next 4 bytes are float
			for (int i = 0; i< Float.SIZE/8; i++)
			interpreter.put(messageBuffer.poll());
			
			interpreter.rewind();
			dataPacket.fData = interpreter.getFloat();	//reading in reverse (LITTLE_ENDIAN order)

			if (messageBuffer.poll().equals(ACTOConst.USB_SEP)) {
				sendByte(ACTOConst.USB_ACK);
				commHub.handleUsbMsg(cmd, dataPacket);
				return true;
			}
			else {
				Log.e(TAG, "USB message corrupt, cleaning up buffer");
				sendByte(ACTOConst.USB_NAK);
				doMessageBufferCleanup();
			}
		}
		return false;
	}

	private boolean readADIdDidByteInt() {		//CMD, ID, dataID(byte), data(byte), data(int), SEP		//for motor_ms
		if (messageBuffer.size() >= 9) {
			Log.i(TAG, "size: " + messageBuffer.size());
			final byte cmd;
			final ACTOData dataPacket = new ACTOData(ACTOData.MOTOR_MS);

			cmd = messageBuffer.poll();
			
			dataPacket.id = messageBuffer.poll();
			dataPacket.dataId = messageBuffer.poll();
			dataPacket.bData = messageBuffer.poll();
			
			Log.i(TAG, "for id: " + dataPacket.id + " and dataid: " + dataPacket.dataId);
			
			interpreter.clear();
			
			//next 4 bytes are int
			for (int i = 0; i< Integer.SIZE/8; i++)
			interpreter.put(messageBuffer.poll());
			
			interpreter.rewind();
			dataPacket.iData = interpreter.getInt(); //reading in reverse (LITTLE_ENDIAN order)
			
			if (messageBuffer.poll().equals(ACTOConst.USB_SEP)) {
				sendByte(ACTOConst.USB_ACK);
				commHub.handleUsbMsg(cmd, dataPacket);
				return true;
			}
			else {
				Log.e(TAG, "USB message corrupt, cleaning up buffer");
				sendByte(ACTOConst.USB_NAK);
				doMessageBufferCleanup();
			}
		}
		return false;
	}

//
// SENDING
//////////////

	//TODO: Check somewhere if AD is valid! z.b type!
	
	//ACK NAK	// format: CMD, SEP
    public boolean sendByte (byte usb_msg) {
		
    	if (usb_msg == ACTOConst.USB_ACK || usb_msg ==ACTOConst.USB_NAK) return true;	//TODO: suppress ACK NAK for the moment
    	
    	try{
			if (isListening){
		    	byte[] sData = {usb_msg, ACTOConst.USB_SEP};
		    	writeByteArray(sData);
				return true;
			}else{
				return false;
			}
		}catch (Exception e){
			Log.e(TAG,"Error sending: " + e);
			e.printStackTrace();
			return false;
		}
    }
    
	//PING SYNC	// format: CMD, ID, SEP
    public boolean sendCmdId (byte usb_msg, ACTOData ad) {	//TODO: better as byte id or both?
		try{
			if (isListening){
		    	byte[] sData = {usb_msg, ad.id, ACTOConst.USB_SEP};
		    	writeByteArray(sData);
				return true;
			}else{
				return false;
			}
		}catch (Exception e){
			Log.e(TAG,"Error sending: " + e);
			e.printStackTrace();
			return false;
		}
    }

	//GETDI & AN	//format: CMD, ID, dataID(byte), SEP
    public boolean sendCmdIdDidEmpty (byte usb_msg, ACTOData ad) {
		try{
			if (isListening){
		    	byte[] sData = {usb_msg, ad.id, ad.dataId, ACTOConst.USB_SEP};
		    	Log.d(TAG, "OUT: COMMAND: " + sData[0] + " for ID: " + sData[1]+ " for DATAID: " + sData[2]+ " SEP: " + sData[3]);
		    	writeByteArray(sData);
				return true;
			}else{
				return false;
			}
		}catch (Exception e){
			Log.e(TAG,"Error sending: " + e);
			e.printStackTrace();
			return false;
		}
    }
    
	//SETDI SETBYTE MOTOR	//format: CMD, ID, dataID(byte), data(byte), SEP
    public boolean sendCmdIdDidByte (byte usb_msg, ACTOData ad) {
		try{
			if (isListening){
		    	byte[] sData = {usb_msg, ad.id, ad.dataId, ad.bData, ACTOConst.USB_SEP};
		    	writeByteArray(sData);
				return true;
			}else{
				return false;
			}
		}catch (Exception e){
			Log.e(TAG,"Error sending: " + e);
			e.printStackTrace();
			return false;
		}
    }
    
	//SETCHAR	//format: CMD, ID, dataID(byte), data(char16), SEP
    public boolean sendCmdIdDidChar (byte usb_msg, ACTOData ad) {
		try{
			if (isListening){
		    	interpreter.clear();
		    	interpreter.putChar(ad.cData);
		    	
//		    	interpreter.rewind();
//		    	Log.i(TAG, "bytes: " + interpreter.get() + ", " + interpreter.get() + ", " + interpreter.get() + ", " + interpreter.get());
		    	
		    	interpreter.rewind();
		    	byte[] sData = {usb_msg, ad.id, ad.dataId, interpreter.get(), interpreter.get(), ACTOConst.USB_SEP};
		    	writeByteArray(sData);
		    	
				return true;
			}else{
				return false;
			}
		}catch (Exception e){
			Log.e(TAG,"Error sending: " + e);
			e.printStackTrace();
			return false;
		}
    }
    
	//SETAN SETINT	//format: CMD, ID, dataID(byte), data(int32), SEP
    public boolean sendCmdIdDidInt (byte usb_msg, ACTOData ad) {
		try{
			if (isListening){
		    	interpreter.clear();
		    	interpreter.putInt(ad.iData);
		    	
//		    	interpreter.rewind();
//		    	Log.i(TAG, "bytesInt: " + interpreter.get() + ", " + interpreter.get() + ", " + interpreter.get() + ", " + interpreter.get());
		    	
		    	interpreter.rewind();
		    	byte[] sData = {usb_msg, ad.id, ad.dataId, interpreter.get(), interpreter.get(), interpreter.get(), interpreter.get(), ACTOConst.USB_SEP};
		    	writeByteArray(sData);
		    	
				return true;
			}else{
				return false;
			}
		}catch (Exception e){
			Log.e(TAG,"Error sending: " + e);
			e.printStackTrace();
			return false;
		}
    }
    
	//SETFLOAT	//format: CMD, ID, dataID(byte), data(float32), SEP
    public boolean sendCmdIdDidFloat (byte usb_msg, ACTOData ad) {
		try{
			if (isListening){

		    	interpreter.clear();
		    	interpreter.putFloat(ad.fData);
		    	
//		    	interpreter.rewind();
//		    	Log.i(TAG, "bytesFloat: " + interpreter.get() + ", " + interpreter.get() + ", " + interpreter.get() + ", " + interpreter.get());
		    	
		    	interpreter.rewind();
		    	byte[] sData = {usb_msg, ad.id, ad.dataId, interpreter.get(), interpreter.get(), interpreter.get(), interpreter.get(), ACTOConst.USB_SEP};
		    	writeByteArray(sData);
				
				return true;
			}else{
				return false;
			}
		}catch (Exception e){
			Log.e(TAG,"Error sending: " + e);
			e.printStackTrace();
			return false;
		}
    }

	//MOTOR_MS	//format: CMD, ID, dataID(byte), data(byte), data(int), SEP
    public boolean sendCmdIdDidByteInt (byte usb_msg, ACTOData ad) {
		try{
			if (isListening){
		    	interpreter.clear();
		    	interpreter.putInt(ad.iData);
		    	
//		    	interpreter.rewind();
//		    	Log.i(TAG, "bytesFloat: " + interpreter.get() + ", " + interpreter.get() + ", " + interpreter.get() + ", " + interpreter.get());
		    	
		    	interpreter.rewind();
		    	byte[] sData = {usb_msg, ad.id, ad.dataId, ad.bData, interpreter.get(), interpreter.get(), interpreter.get(), interpreter.get(), ACTOConst.USB_SEP};
		    	writeByteArray(sData);
		    	
				return true;
			}else{
				return false;
			}
		}catch (Exception e){
			Log.e(TAG,"Error sending: " + e);
			e.printStackTrace();
			return false;
		}
    }

    //TODO: wie ACTODATA verwenden? fr alle Daten? nur als TUIO, oder ad.type = DATA_INT, ad.iData =0, wie dann mit iArray[] in TUIO??
    
	

	// use this to actually write to USB
	private void writeByteArray(byte[] data) {
        synchronized (writeBuffer) {
            writeBuffer.put(data);
        }
    }
    
	
	//
	//	CLEANING
	////////////////	
	
  	public void closeTransmitter() {
  		Log.d(TAG,"Closing Transmitter...");
  	    reconnect = false; //stop running Thread loop trying to wait for new clients
  	    isListening = false; //stop running Thread loop listening on data
  	    if(driver != null) {
  	    	try {
				driver.close();
			} catch (IOException e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
			}
  	    }
  	    driver = null;
  	    usbStateChange(ACTOConst.STATE_CLOSED);
  	}
}
