package at.ac.tuwien.ims.acto_framework;

/**
 * WifiTransmitter:
 * 		handles communication between PC and Android
 * 
 * @author Emanuel Vonach
 * 
 * based on code by Kaltenriner Christoph, Leichtfried Michael
 */

import java.io.DataInputStream;
import java.io.DataOutputStream;
import java.io.IOException;
//import java.net.InetAddress;
//import java.net.NetworkInterface;
import java.net.ServerSocket;
import java.net.Socket;

//import android.net.wifi.WifiManager;
import android.util.Log;

//TODO: proper synchronization throughout the class!!
public class WifiTransmitter implements Runnable 
{
	private final String TAG = this.getClass().getSimpleName();
	
	//Communication 
	private ServerSocket server;
	private Socket client;
	private DataInputStream dis = null;
	private DataOutputStream dos = null;
    
    //calling commHub
    private TUIOCommHub commHub;
    
    //Threadcontrol
    private boolean isListening = false;
    private boolean reconnect = true;

    
    /**
     * WifiTransmitter: open ServerSocket and listen for clients 
     * 
     * @param int port: port number for connection
     * @param TUIOCommHub commHub: instance of caller, to return results
     * @throws IOException
     */
	public WifiTransmitter(int port, TUIOCommHub commHub) throws IOException
	{
		Log.d(TAG,"starting WIFITransmitter");
		server = new ServerSocket(port);
		Log.d(TAG, "Reuseaddress: " + server.getReuseAddress());
		this.commHub = commHub;
	}
	
	//listen for clients and on input
	public void run() 
	{
		//if connection gets lost and reconnect is set true, retry to accept new clients
		while(reconnect)
		{
			//wait for client
			Log.d(TAG, "waiting for clients...");
			//tell calling class that we are ready to accept clients
			wifiStateChange(ACTOConst.STATE_PENDING);
			try 
			{
				//locks thread as long as there is no client to accept
				client = server.accept();
				wifiStateChange(ACTOConst.STATE_CONNECTED);
				//TODO: check thread security
				isListening = true;
			} 
			
			catch (IOException e)
			{
				//wifiStateChange(TUIO_CommHub.STATE_PENDING);
				isListening = false;
				Log.d(TAG, "Connection canceled");
				//e.printStackTrace();
				continue;  //most likely the attempt to accept clients got interrupted, ignore following code and try again
			}
		
			//Connection is accepted now listen for input
			Log.d(TAG,"Client connected");

			try{
				// set up printWriter
				dos = new DataOutputStream(client.getOutputStream());
				dis = new DataInputStream(client.getInputStream());
			}
			catch (Exception e)
			{
				// something bad happened, better cleanup everything
				Log.e(TAG,"EXEPTION: Open Listener" + e);
				e.printStackTrace();
				closeTransmitter();
			}
			
//			sendByte(ACTO_Const.WL_ACK);	//handshake if necessary

			//listen on input
			while (isListening) 
			{				
				byte input;
				try
				{
					input = dis.readByte();					
				}
				catch(IOException e)
				{
					Log.e(TAG, "unable to read Byte");
					//e.printStackTrace();
					isListening = false; //stop reading and try to reconnect
					cleanupStreams();
					break; 
				}
				
//				//TODO: what does 127 mean?
//					if (input == 127) break;
				
				switch (input){
					// ping/handshake
					case (ACTOConst.WI_ACK):			
					// read simple CMD messages
//					case (ACTOConst.WI_FULL_STATE): case (ACTOConst.WI_FULL_EXT):
//					case (ACTOConst.WI_FULL_TRANS): case (ACTOConst.WI_FULL_ROT): 
					case (ACTOConst.WI_ALL_POSE):
					case (ACTOConst.WI_GET_PLSIZE):
					case (ACTOConst.WI_GET_ST_USB): case (ACTOConst.WI_GET_ST_APBRC): case (ACTOConst.WI_GET_ST_WIFI):
					case (ACTOConst.WI_GET_ST_WIBRC): case (ACTOConst.WI_GET_ST_CAL):
						readSimpleCmd(input);
						break;	
					
					// read CMD + int messages
					case (ACTOConst.WI_SET_ST_USB): case (ACTOConst.WI_SET_ST_APBRC): case (ACTOConst.WI_SET_ST_WIFI):
					case (ACTOConst.WI_SET_ST_WIBRC): case (ACTOConst.WI_SET_ST_CAL):			
						readADState(input);	//TODO: overhead to use ACTOData for anything but trans,rot,pose,etc, separate handleWifiCmd? overload?
						//TODO: probably use byte for state, but then separate msg for port
						break;
					
					// read CMD + int[] messages
					case (ACTOConst.WI_SET_PLSIZE):
						readADRes(input);
						break;
					
					// read CMD + ID messages
					//case (ACTOConst.WI_GET_STATE):
					case (ACTOConst.WI_GET_POSE):
					case (ACTOConst.WI_GET_TRANS): case (ACTOConst.WI_GET_ROT):
						readADId(input);
						break;	
					
					// read CMD + ID + translation messages
					case (ACTOConst.WI_SET_TRANS):
						readADIdTrans(input);
						break;	
					
					// read CMD + ID + rotation messages
					case (ACTOConst.WI_SET_ROT):
						readADIdRot(input);
						break;	
					
					// read CMD + ID + pose messages
					case (ACTOConst.WI_SET_POSE):
						readADIdPose(input);
						break;	
					
//					case (ACTOConst.WI_SET_STATE): 
						
					case (ACTOConst.WI_GET_EXT): case (ACTOConst.WI_SET_EXT):
						readUsbMsg(input);
						break;	
					
					// message unknown					
					default:
						Log.e(TAG, "INVALID COMMAND: " + input);
						sendByte(ACTOConst.WI_NAK);
						break;
				}
			}
		}
	}

	//function for thread state change
	private void wifiStateChange(int state) {
		commHub.onWifiStateChange(state);
	}
	

	
	
	
	//
	//	READING
	///////////////////////
	
	//TODO: how to proceed with incomplete messages? e.g. return ACTOData.command = NAK
	// should be done like UsbSerialComm, readCmds should just return AD to be used twoways, Usb & Wifi reader could be merged, message target could be introduced
	
//	if blocking poses a problem
//	private byte readByteNonBlock() throws IOException {
//		byte bt;
//		if (dis.available() > 0) {
//			bt = dis.readByte();
//		}
//		else {
//			throw new IOException();
//		}
//		return bt;
//	}
	
	

	
	/**
	 * Reads from WiFi: byte command,
	 * 	sends command & data to commHandler and WI_ACK
	 * @param byte input:	the first byte of the datastream = command
	 */
	private void readSimpleCmd(byte input)
	{
		//sendByte(ACTOConst.WI_ACK);		//TODO: perhaps wait for boolean from commHub, call commHub from Thread, use rather as reader, independantly from direction!
		commHub.handleWifiMsg(input, null);
	}
	

	/**
	 * Reads from WiFi, byte command, int state
	 * 	sends command & data to commHandler and WI_ACK/WI_NAK
	 * @param byte input:	the first byte of the datastream = command
	 */
	private void readADState(byte input) {
		
		final ACTOData dataPacket = new ACTOData(ACTOData.STATE);
		try
		{
			dataPacket.state = dis.readInt();
			//sendByte(ACTOConst.WI_ACK);
			commHub.handleWifiMsg(input, dataPacket);
		}
		catch(IOException e)
		{
			sendByte(ACTOConst.WI_NAK);
			Log.e(TAG, "unable to read ACTOData package");
			e.printStackTrace();
		}
	}

	
	/**
	 * Reads from WiFi, byte command, int[3] resolution
	 *	sends command & data to commHandler and WI_ACK/WI_NAK
	 * @param byte input:	the first byte of the datastream = command
	 */
	private void readADRes(byte input) {
		final ACTOData dataPacket = new ACTOData(ACTOData.RES);
		try
		{
			int[] res = new int[3];
			res[0] = dis.readInt();
			res[1] = dis.readInt();
			res[2] = dis.readInt();
			dataPacket.resolution = res;
			//sendByte(ACTOConst.WI_ACK);
			commHub.handleWifiMsg(input, dataPacket);
		}
		catch(IOException e)
		{
			sendByte(ACTOConst.WI_NAK);
			Log.e(TAG, "unable to read ACTOData package");
			e.printStackTrace();
		}
	}


	/**
	 * Reads from WiFi, byte command, byte id
	 *	sends command & data to commHandler and WI_ACK/WI_NAK
	 * @param byte input:	the first byte of the datastream = command
	 */
	private void readADId(byte input) {
		final ACTOData dataPacket = new ACTOData(ACTOData.ID);
		try
		{
			dataPacket.id = dis.readByte();
			//sendByte(ACTOConst.WI_ACK);
			commHub.handleWifiMsg(input, dataPacket);
		}
		catch(IOException e)
		{
			sendByte(ACTOConst.WI_NAK);
			Log.e(TAG, "unable to read ACTOData package");
			e.printStackTrace();
		}
	}

	
	/**
	 * Reads from WiFi,  byte command, byte id, float[3] translation
	 *	sends command & data to commHandler and WI_ACK/WI_NAK
	 * @param byte input:	the first byte of the datastream = command
	 */
	private void readADIdTrans(byte input)
	{
		final ACTOData dataPacket = new ACTOData(ACTOData.TRANS);
		try
		{
			dataPacket.id = dis.readByte();
			float[] trans = new float[3];
			trans[0]  = dis.readFloat();
			trans[1]  = dis.readFloat();
			trans[2]  = dis.readFloat();
			dataPacket.translation = trans;
			//sendByte(ACTOConst.WI_ACK);
			commHub.handleWifiMsg(input, dataPacket);
		}
		catch(IOException e)
		{
			sendByte(ACTOConst.WI_NAK);
			Log.e(TAG, "unable to read ACTOData package");
			e.printStackTrace();
		}
	}


	/**
	 * Reads from WiFi,  byte command, byte id, float[4] rotation
	 *	sends command & data to commHandler and WI_ACK/WI_NAK
	 * @param byte input:	the first byte of the datastream = command
	 */
	private void readADIdRot(byte input)
	{
		final ACTOData dataPacket = new ACTOData(ACTOData.ROT);
		try
		{
			dataPacket.id = dis.readByte();
			float[] rot = new float[4];
			rot[0]  = dis.readFloat();
			rot[1]  = dis.readFloat();
			rot[2]  = dis.readFloat();
			rot[3]  = dis.readFloat();
			dataPacket.rotation = rot;
			//sendByte(ACTOConst.WI_ACK);
			commHub.handleWifiMsg(input, dataPacket);
		}
		catch(IOException e)
		{
			sendByte(ACTOConst.WI_NAK);
			Log.e(TAG, "unable to read ACTOData package");
			e.printStackTrace();
		}
	}


	/**
	 * Reads from WiFi,  byte command, byte id, float[3] translation, float[4] rotation
	 *	sends command & data to commHandler and WI_ACK/WI_NAK
	 * @param byte input:	the first byte of the datastream = command
	 */
	private void readADIdPose(byte input)
	{
		final ACTOData dataPacket = new ACTOData(ACTOData.POSE);
		try
		{
			dataPacket.id = dis.readByte();
			float[] trans = new float[3];
			trans[0]  = dis.readFloat();
			trans[1]  = dis.readFloat();
			trans[2]  = dis.readFloat();
			float[] rot = new float[4];
			rot[0]  = dis.readFloat();
			rot[1]  = dis.readFloat();
			rot[2]  = dis.readFloat();
			rot[3]  = dis.readFloat();
			dataPacket.translation = trans;
			dataPacket.rotation = rot;
			//sendByte(ACTOConst.WI_ACK);
			commHub.handleWifiMsg(input, dataPacket);
		}
		catch(IOException e)
		{
			sendByte(ACTOConst.WI_NAK);
			Log.e(TAG, "unable to read ACTOData package");
			e.printStackTrace();
		}
	}


	/**
	 * Reads from WiFi,  byte command, byte id, usbMsg
	 *  usbMsg according to protocol
	 *	sends command & data to commHandler and WI_ACK/WI_NAK
	 * @param byte input:	the first byte of the datastream = command
	 */
	private void readUsbMsg(byte input) {
		final ACTOData dataPacket = new ACTOData();
		
		try
		{
			final byte usbMsg = dis.readByte();
			dataPacket.id = dis.readByte();
			
			//TODO: should be rewritten with UsbSerialComm, unnecessary duality
			switch (usbMsg) {
//				case ACTOConst.USB_ACK:	//makes no sense here?
//					break;
	
				case ACTOConst.USB_PING: case ACTOConst.USB_SYNC:
					dataPacket.type = ACTOData.ID;
					break;
					
				case ACTOConst.USB_GET_DI_UNIT: case ACTOConst.USB_GET_AN_UNIT: case ACTOConst.USB_GET_BYTE:
				case ACTOConst.USB_GET_CHAR: case ACTOConst.USB_GET_INT: case ACTOConst.USB_GET_FLOAT:
					dataPacket.type = ACTOData.DATA_EMPTY;
					dataPacket.dataId = dis.readByte();
					break;
					
				//case ACTOConst.USB_MOTOR:
				case ACTOConst.USB_SET_DI_UNIT: case ACTOConst.USB_SET_BYTE:
					dataPacket.type = ACTOData.DATA_BYTE;
					dataPacket.dataId = dis.readByte();
					dataPacket.bData = dis.readByte();
					break;
					
				case ACTOConst.USB_SET_CHAR:
					dataPacket.type = ACTOData.DATA_CHAR;
					dataPacket.dataId = dis.readByte();
					dataPacket.cData = dis.readChar();
					break;
					
				case ACTOConst.USB_SET_AN_UNIT: //TODO: should be removed, only used as answer!
				case ACTOConst.USB_SET_PWM_UNIT: case ACTOConst.USB_SET_INT:
					dataPacket.type = ACTOData.DATA_INT;
					dataPacket.dataId = dis.readByte();
					dataPacket.iData = dis.readInt();
					break;
	
				case ACTOConst.USB_SET_FLOAT:
					dataPacket.type = ACTOData.DATA_FLOAT;
					dataPacket.dataId = dis.readByte();
					dataPacket.fData = dis.readFloat();
					break;
					
				case ACTOConst.USB_MOTOR_MS:
					dataPacket.type = ACTOData.MOTOR_MS;
					dataPacket.dataId = dis.readByte();
					dataPacket.bData = dis.readByte();
					dataPacket.iData = dis.readInt();
					break;

				
				default:
					Log.e(TAG, "USB message corrupt");
					throw new IOException(); 
			}
			
			//sendByte(ACTOConst.WI_ACK);
			commHub.handleWifiMsg(input, usbMsg, dataPacket);
			
		}
		catch(IOException e)
		{
			sendByte(ACTOConst.WI_NAK);
			Log.e(TAG, "unable to read ACTOData package");
			e.printStackTrace();
		}
	}
	
	
	//
	// SENDING
	//////////////
	
	
	/**
	 * Sends simple byte to Wifi
	 * @param byte bt:	single byte to send
	 * @return boolean: true if successful
	 */
	public boolean sendByte(byte bt){
		try{
			if (isListening){
				dos.writeByte(bt);
				return true;
			}else{
				return false;
			}
		}catch (Exception e){
			Log.e(TAG,"Error sending: " + e);
			e.printStackTrace();
			return false;
		}
	}
	
//	/**
//	 * Sends ACTOData to Wifi, byte command
//	 * @param ACTOData ad:	ACTOData packet with: byte command
//	 * @return boolean:	true if successful
//	 */
//	public boolean sendADByte(ACTOData ad){
//		
//		try{
//			if (isListening){
//				dos.writeByte(ad.command);
//				return true;
//			}else{
//				return false;
//			}
//		}catch (Exception e){
//			Log.e(TAG,"Error sending: " + e);
//			e.printStackTrace();
//			return false;
//		}
//	}
		

	/**
	 * Sends state data to Wifi
	 * @param byte wi_msg:	command byte
	 * @param int state:	state as int
	 * @return boolean:	true if successful
	 */
	public boolean sendState(byte wi_msg, int state)
	{
		try{
			if (isListening){
				dos.writeByte(wi_msg);
				dos.writeInt(state);
				return true;
			}else{
				return false;
			}
		}catch (Exception e){
			Log.e(TAG,"Error sending: " + e);
			e.printStackTrace();
			return false;
		}
	}
	
	/**
	 * Sends resolution to Wifi
	 * @param byte wi_msg:	command byte
	 * @param int[] res:	resolution x,y,z as int
	 * @return boolean:	true if successful
	 */
	public boolean sendRes(byte wi_msg, int[] res)
	{
		try{
			if (isListening){
				dos.writeByte(wi_msg);
				dos.writeInt(res[0]);
				dos.writeInt(res[1]);
				dos.writeInt(res[2]);
				return true;
			}else{
				return false;
			}
		}catch (Exception e){
			Log.e(TAG,"Error sending: " + e);
			e.printStackTrace();
			return false;
		}
	}
	
//	/**
//	 * Sends ACTOData to Wifi, byte command, byte id
//	 * @param ACTOData ad:	ACTOData packet with: byte command, byte id
//	 * @return boolean:	true if successful
//	 */
//	public boolean sendId(byte wi_msg, byte id)
//	{
//		try{
//			if (isListening){
//				dos.writeByte(wi_msg);
//				dos.writeByte(id);
//				return true;
//			}else{
//				return false;
//			}
//		}catch (Exception e){
//			Log.e(TAG,"Error sending: " + e);
//			e.printStackTrace();
//			return false;
//		}
//	}
	
//	/**
//	 * Sends ACTOData to Wifi, byte command, byte id, int iValue
//	 * @param ACTOData ad:	ACTOData packet with: byte command, byte id, int iValue
//	 * @return boolean:	true if successful
//	 */
//	public boolean sendADIdState(byte wi_msg, ACTOData ad)
//	{
//		try{
//			if (isListening){
//				dos.writeByte(wi_msg);
//				dos.writeByte(ad.id);
//				dos.writeInt(ad.state);
//				return true;
//			}else{
//				return false;
//			}
//		}catch (Exception e){
//			Log.e(TAG,"Error sending: " + e);
//			e.printStackTrace();
//			return false;
//		}
//	}

//	/**
//	 * Sends ACTOData to Wifi, byte command, byte id, float fValue
//	 * @param ACTOData ad:	ACTOData packet with: byte command, byte id, float fValue
//	 * @return boolean:	true if successful
//	 */
//	public boolean sendADIdExt(byte wi_msg, ACTOData ad)
//	{
//		try{
//			if (isListening){
//				dos.writeByte(ad.command);
//				dos.writeByte(ad.id);
//				dos.writeFloat(ad.fValue);
//				return true;
//			}else{
//				return false;
//			}
//		}catch (Exception e){
//			Log.e(TAG,"Error sending: " + e);
//			e.printStackTrace();
//			return false;
//		}
//	}
	
	/**
	 * Sends ACTOData translation to Wifi
	 * @param byte wi_msg:	command byte
	 * @param ACTOData ad:	ACTOData packet with: byte id, float[3] translation
	 * @return boolean:	true if successful
	 */
	public boolean sendADIdTrans(byte wi_msg, ACTOData ad)
	{
		try{
			if (isListening){
				dos.writeByte(wi_msg);
				dos.writeByte(ad.id);
				dos.writeFloat(ad.translation[0]);
				dos.writeFloat(ad.translation[1]);
				dos.writeFloat(ad.translation[2]);
				return true;
			}else{
				return false;
			}
		}catch (Exception e){
			Log.e(TAG,"Error sending: " + e);
			e.printStackTrace();
			return false;
		}
	}


	/**
	 * Sends ACTOData rotation to Wifi
	 * @param byte wi_msg:	command byte
	 * @param ACTOData ad:	ACTOData packet with: byte id, float[4] rotation
	 * @return boolean:	true if successful
	 */
	public boolean sendADIdRot(byte wi_msg, ACTOData ad)
	{
		try{
			if (isListening){
				dos.writeByte(wi_msg);
				dos.writeByte(ad.id);
				dos.writeFloat(ad.rotation[0]);
				dos.writeFloat(ad.rotation[1]);
				dos.writeFloat(ad.rotation[2]);
				dos.writeFloat(ad.rotation[3]);
				return true;
			}else{
				return false;
			}
		}catch (Exception e){
			Log.e(TAG,"Error sending: " + e);
			e.printStackTrace();
			return false;
		}
	}


	/**
	 * Sends ACTOData pose to Wifi
	 * @param byte wi_msg:	command byte
	 * @param ACTOData ad:	ACTOData packet with: byte id, float[3] translation, float[4] rotation
	 * @return boolean:	true if successful
	 */
	public boolean sendADIdPose(byte wi_msg, ACTOData ad)
	{
		try{
			if (isListening){
				dos.writeByte(wi_msg);
				dos.writeByte(ad.id);
				
				dos.writeFloat(ad.translation[0]);
				dos.writeFloat(ad.translation[1]);
				dos.writeFloat(ad.translation[2]);

				dos.writeFloat(ad.rotation[0]);
				dos.writeFloat(ad.rotation[1]);
				dos.writeFloat(ad.rotation[2]);
				dos.writeFloat(ad.rotation[3]);
				
				//Log.d(TAG, "Data: " + ad.rotation[0] + " " +  ad.rotation[1] + " " + ad.rotation[2] + " " + ad.rotation[3]);
				return true;
			}else{
				return false;
			}
		}catch (Exception e){
			Log.e(TAG,"Error sending: " + e);
			e.printStackTrace();
			return false;
		}
	}
	
	
	/**
	 * Sends usbMsg to Wifi
	 * @param byte usbMsg:	usbMsg to send
	 * @param ACTOData ad:	ACTOData packet with data acording to protocol
	 * @return boolean:	true if successful
	 */
	public boolean sendUsbMsg(byte usbMsg, ACTOData ad) {
		try{
			if (isListening){

				//TODO: should be rewritten with UsbSerialComm, unnecessary duality
				switch (usbMsg) {
					case ACTOConst.USB_ACK:	//makes no sense here?
						dos.writeByte(ACTOConst.WI_SET_EXT);
						dos.writeByte(usbMsg);
						break;
		
					case ACTOConst.USB_PING: case ACTOConst.USB_SYNC:
						dos.writeByte(ACTOConst.WI_SET_EXT);
						dos.writeByte(usbMsg);
						dos.writeByte(ad.id);
						break;
						
						//makes no sense in that direction?
//					case ACTOConst.USB_GET_DI_UNIT: case ACTOConst.USB_GET_AN_UNIT: case ACTOConst.USB_GET_BYTE:
//					case ACTOConst.USB_GET_CHAR: case ACTOConst.USB_GET_INT: case ACTOConst.USB_GET_FLOAT:
//
//						break;
						
					//case ACTOConst.USB_MOTOR: //makes no sense in that direction?
					case ACTOConst.USB_SET_DI_UNIT: case ACTOConst.USB_SET_BYTE:
						dos.writeByte(ACTOConst.WI_SET_EXT);
						dos.writeByte(usbMsg);
						dos.writeByte(ad.id);
						dos.writeByte(ad.dataId);
						dos.writeByte(ad.bData);
						break;
						
					case ACTOConst.USB_SET_CHAR:
						dos.writeByte(ACTOConst.WI_SET_EXT);
						dos.writeByte(usbMsg);
						dos.writeByte(ad.id);
						dos.writeByte(ad.dataId);
						dos.writeChar(ad.cData);
						break;
						
					//case ACTOConst.USB_SET_PWM_UNIT: //makes no sense in that direction?
					case ACTOConst.USB_SET_AN_UNIT: case ACTOConst.USB_SET_INT:
						dos.writeByte(ACTOConst.WI_SET_EXT);
						dos.writeByte(usbMsg);
						dos.writeByte(ad.id);
						dos.writeByte(ad.dataId);
						dos.writeInt(ad.iData);
						break;
		
					case ACTOConst.USB_SET_FLOAT:
						dos.writeByte(ACTOConst.WI_SET_EXT);
						dos.writeByte(usbMsg);
						dos.writeByte(ad.id);
						dos.writeByte(ad.dataId);
						dos.writeFloat(ad.fData);
						break;
					
//					case ACTOConst.USB_MOTOR_MS: //makes no sense in that direction?
//						dos.writeByte(ACTOConst.WI_SET_EXT);
//						dos.writeByte(usbMsg);
//						dos.writeByte(ad.id);
//						dos.writeByte(ad.dataId);
//						dos.writeByte(ad.bData);
//						dos.writeInt(ad.iData);
//						break;
						
					default:
						Log.e(TAG, "USB message corrupt");
						throw new IOException(); 
				}
				return true;
			}else{
				return false;
			}
		}catch (Exception e){
			Log.e(TAG,"Error sending: " + e);
			e.printStackTrace();
			return false;
		}
	}
	
	
	//
	//	CLEANING
	////////////////
	
	/**
	 * Close Communication and send status to Android
	 * @throws IOException
	 */
	public void closeTransmitter()
	{
		Log.d(TAG,"Closing Transmitter...");

		//TODO: probably inform client, if existing?
		
		reconnect = false;   //stop running Thread loop trying to wait for new clients
		isListening = false;  //stop running Thread loop listening on data

		//TODO: perhaps better do notification after cleanup?
		wifiStateChange(ACTOConst.STATE_CLOSED);
		cleanupTransmitter();
	}
	
	private void cleanupTransmitter()
	{
		try
		{
			Log.d(TAG, "cleaning up transmitter...");
			
			cleanupStreams();
			server.close();
			if(client != null)
				client.close();	
		}
		catch(IOException e)
		{
			Log.e(TAG, "failed closing Network connection", e);
			e.printStackTrace();
		}
		finally
		{
			client = null;
			server = null;
		}
	}
	
	// free datastreams for use with new client
	private void cleanupStreams()
	{
		try
		{
			if(dis != null)
				dis.close();
			if(dos != null)
				dos.close();
		}
		catch(IOException e)
		{
			Log.e(TAG, "error closing Streams");
			e.printStackTrace();
		}
		finally
		{
			dis = null;
			dos = null;
		}
	}
	
	
}
